/* 
	Create an id for a radio button or checkbox by concatenating the 
	the groupId with newValue and then replacing the spaces with 
	underscores '_'
*/
function makeId(groupId, newValue) {
	var newId = groupId + "_" + newValue;
  return newId.replace(new RegExp(' ', 'g'), '_');
}

/* 
	Create a text node from a newText string 
*/
function makeTextNode(newText) {
	return document.createTextNode(newText);
}

/* 
	Make an HTML <label> element with the specified labelText and newId
	
	newId is an optional parameter	
*/	
function makeLabel(labelText, newId) {
	var newLabel = document.createElement("label");
	if (newId !== undefined && newId != "") {
		newLabel.setAttribute("for", newId);
	}
	newLabel.appendChild(makeTextNode(labelText));
	return newLabel;
}

/* 
	Make an HTML <input> element of the specified type with 
	optional labelText, newId, newName, and newValue.
	
	By default required is false. If you set required to true, class="required" will be added. 
	
	If type is button, the Bootstrap CSS class btn and btn-primary are added
*/
function makeInput(type, labelText, newId, newName, newValue, required = false) {
	var newInput = document.createElement("input");
	newInput.setAttribute("type", type);
	if (newId !== undefined && newId != "") {
		newInput.setAttribute("id", newId);
	}
	if (newName !== undefined && newName != "") {
		newInput.setAttribute("name", newName);
	}
	if (newValue !== undefined && newValue != "") {
		newInput.setAttribute("value", newValue);
	}
	if (required) {
		newInput.classList.add("required");
	}
	if (type == "button") {
		newInput.classList.add("btn");
		newInput.classList.add("btn-primary");
	}	
	if (labelText !== undefined && labelText != "") { 	 	
		var newLabel = makeLabel(labelText, newId);
		// If radio or checkbox insert the newInput before the labelText node of the newLabel
		if (type == "radio"  || type == "checkbox") {
			newLabel.insertBefore(newInput, newLabel.firstChild);
			return newLabel;
		}
		else {
		  var newDiv = document.createElement("div");
			newDiv.appendChild(newLabel);
			newDiv.appendChild(newInput);
			return newDiv;
		}
	}
	else {
		return newInput;
	}
}

/* 
	Make a group of radio buttons or checkboxes 

	The buttons or checkboxes are in a <span> element
	
	newID is used as the id of the span
		
	If labelText is defined, a <label> element is created and 
	the label and span are put inside of a <div> element.
	
	If type is checkbox then "[]" is appended to newID
	
	newID is used as the name of each radio button or checkbox.
	
	Each radio button or checkbox id is generated using 
	the makeId function (see above)
*/
function makeGroup(type, arrayOfValues, labelText, newId, required = false) {
	var newSpan = document.createElement('span');
	if (newId !== undefined && newId != "") {
		if (type == "checkbox") {
			newId = newId + "[]";
		}
		newSpan.setAttribute("id", newId);
	}
	if (required) {
		newSpan.setAttribute("class", "required");
	}
	for (var v = 0; v < arrayOfValues.length; v++) {
		var value = arrayOfValues[v];
		var itemId = "";
		if (newId !== undefined && newId != "") {
			itemId = makeId(newId, value);
		}
		var newInput = makeInput(type, value, itemId, newId, value);
		newSpan.appendChild(newInput);
	}
	if (labelText !== undefined && labelText != "") {
		var newDiv = document.createElement("div");
		newDiv.appendChild(makeLabel(labelText, newId));
		newDiv.appendChild(newSpan);
		return newDiv;
	}
	else {
		return newSpan;
	}
}

/* 
	When the AJAX response is received/loaded, 
	the server-side script output will be put in the listArea's innerHTML 
*/
function responseReceived() {
	if (this.status === 200) {
		listArea.innerHTML = this.response;
  } 
  else {
		listArea.innerHTML = "The request failed, status: " + this.status + " " + this.statusText;
  }
}

/* 
	Encode the specified dataStructure into a JSON string, 
	send it to the specified URL and
	call responseReceived when the response is ready/loaded
*/
function jsonSend(dataStructure, url) {
	var jsonData = JSON.stringify(dataStructure);
	var request = new XMLHttpRequest();
	request.addEventListener("load", responseReceived);
	request.open("POST", url, true);
	request.setRequestHeader("Content-type", "application/json");
	request.send(jsonData);		
}

/* 
	Return a random number between min and max
	
	Hints:
		- Use Math.floor and Math.random
*/
function rand(min, max) {
	var num = Math.floor(Math.random() * (max - min + 1)) + min;
	return num;
}

/* 
	Return a random rgb color string with value between hi and low
	
	Example: rgb(99,126,81)
	
	Hints:
		- The numeric values are random values between 0 and 255
		- Use the rand function you defined 
		- Use the concatenation + operator to build the string 
	
*/
function randomColor(low,hi) {
	var col = "rgb("+rand(low,hi)+","+rand(low,hi)+","+rand(low,hi)+")";
	return col;
}


/* 
	Pops a random value from the selectedItems array
	
	Help:  
		- Make sure selectedItems has length greater than zero
		- Use the rand function (see above) to get a random index between 0 and selectedItems.length
		- Using a local variable, save the value of the array at that random index
		- Remove the array item at the index using the JavaScript's array splice function
		- Return the saved value 
*/
function popRandomValue() {
	if(selectedItems.length > 0){
		var num = rand(0, selectedItems.length - 1);
		var val = selectedItems[num];
		selectedItems.splice(num,1);
		return val;
	}
}


/*
	Updates a button with a new random value from the selectedItems array
	and give the button a random color
	
*/
function updateButton(button) {
	var val = popRandomValue();
	button.setAttribute("value",val);
	button.style.backgroundColor = randomColor(0,255);
}


/*
	This function is called each time itemOne or itemTwo is clicked.
	
	It will record the winner, i.e., the button the user clicked, and 
	the loser, i.e., the button the user did not click.
	
	If itemOne was clicked it will be passed as the winner and itemTwo 
	will be passed as the loser, or vice versa if itemTwo was clicked

	We will send the value of the winner and loser using AJAX/JSON 
	as an object with two fields
	
*/
function recordRanking(winner, loser) {
	var ranking = new Object();
	ranking.winner = winner.value;
	ranking.loser = loser.value;
	jsonSend(ranking, "http://s934939.sienasellbacks.com/project3/addrankings.php");
	if (selectedItems == null || selectedItems.length == 0){ 
		var pop = btnArea.parentNode;
		pop.removeChild(btnArea);
		messageArea.innerHTML = "All Done.  Reload the page to start over.";
		
	}
	else{
		updateButton(loser);
	}
}


/*
	Starts the ranking process
	
	Only do this if the selectedItems array has 2 or more items
	
	Use the jsonSend function to send the selectedItems array to the following server-side script:
	http://breimer.sienacs.com/courses/csis-390-s17/projects/project3/additems.php
	
*/	
function rankSelectedItems() {
	if(selectedItems.length >= 2){
		var pop = nextButton.parentNode;
		pop.removeChild(nextButton);
		jsonSend(selectedItems,"http://s934939.sienasellbacks.com/project3/additems.php");
		messageArea.innerHTML = "Which of the items below do you like better? <b>Click one</b>:"; 
		itemOne = makeInput("button", "", "", "");
		itemTwo = makeInput("button", "", "", "");
		updateButton(itemOne);
		updateButton(itemTwo);
		btnArea.appendChild(itemOne);
		btnArea.appendChild(itemTwo);
		
		itemOne.addEventListener("click", function() {
		recordRanking(itemOne, itemTwo);
		});	
	
		itemTwo.addEventListener("click", function() {
		recordRanking(itemTwo, itemOne);
		});
	}
}

/* 
	Update the innerHTML of the messageArea to indicate which items are selected.

	If less than 2 items are selected display "You must select two or more items"

	If two items are selected, display "You have selected Item1 and Item2"

	If three or more items are selected, display  the items if the following format:
	"You have selected Item1, Item2, ..., ItemN-1 and Item N"

*/
function updateMessageArea() {
	//alert(selectedItems);
	var items = [];
	if(selectedItems.length > 2){
		for(i=0; i<selectedItems.length; i++){
			items.push(selectedItems[i]);
		}
		messageArea.innerHTML = "You have selected " + items + ", ";
	}
	else if(selectedItems.length == 2){
		messageArea.innerHTML = "You have selected " + selectedItems[0] + " and " + selectedItems[1];
	}
	else{
		messageArea.innerHTML = "You must select two or more items";
	}
}


/* 
	This function gets called when any of the checkboxes (i.e., <input type="checkbox">) are changed
	
*/
function addSelectedItem() {
	if(this.checked == true){
		this.parentNode.setAttribute("class","highlight");
		selectedItems.push(this.value);
	}
	else{
		this.parentNode.removeAttribute("class");
		selectedItems.splice(selectedItems.indexOf(this.value),1);
	}
	updateMessageArea();
}


/* 
	Add eventListeners to all the checkboxes so that when a checkbox is changed 
	the function addSelectedItem (see above) is called 
	
*/
function addCheckboxListeners() {
	var items = document.querySelectorAll("input[type = checkbox]");
	for(i=0; i<items.length; i++)
	{
			items[i].addEventListener("change",addSelectedItem);
	}
}


/* 

	Below is the main program which will set up the user interface and
	all the event listeners which will call the functions above.

 */

var listOfItems = ["Giants","Jets","Mets","Yankees","Bills","Knicks","Liberty","Nets","Rangers","Islanders","NYC FC","Red Bulls"];
var selectedItems = [];

var itemOne = null;
var itemTwo = null;
var bdy = document.querySelector("body");

var newDiv = document.createElement("div");
newDiv.setAttribute("id","messageArea");
newDiv.innerHTML = "Select ypur favorite New York sports team and click Next";
bdy.appendChild(newDiv);

var btnArea = document.createElement("div");
btnArea.setAttribute("id","btnArea");
var nextButton = makeInput("button", "", "", "", "Next", required = false);
nextButton.addEventListener("click",rankSelectedItems);
btnArea.appendChild(nextButton);
bdy.appendChild(btnArea);

var list = document.createElement("div");
var listArea = makeGroup("checkbox", listOfItems);
list.appendChild(listArea);
bdy.appendChild(list);
addCheckboxListeners();
